@@ -102,9 +102,10 @@ sub multi {
 
 sub multi_p {
   my ($self, @p) = @_;
+  Carp::croak('multi_p(@promises) syntax is not supported anymore. Use promise chaining instead.')
+    if @p;
   $self->{txn} = 'default';
-  my $p = $self->connection->write_p('MULTI');
-  return @p ? $p->then(sub { Mojo::Promise->all(@p) }) : $p;
+  return $self->connection->write_p('MULTI');
 }
 
 sub _add_method {
@@ -1081,21 +1082,26 @@ See L</multi_p>.
   $res     = $db->multi;
   $db      = $db->multi(sub { my ($db, $err, $res) = @_ });
   $promise = $db->multi_p;
-  $promise = $db->multi_p(@promises);
 
 Mark the start of a transaction block. Commands issued after L</multi> will
 automatically be discarded if C<$db> goes out of scope. Need to call
 L</exec> to commit the queued commands to Redis.
 
-When calling L</multi_p>, you can directly pass in the new instructions:
+NOTE: the previously supported C<multi_p(@promises)> syntax has been removed,
+because it did not work as expected. See
+L<https://github.com/jhthorsen/mojo-redis/issues/68> for details.
+When L</multi_p> gets called with non-zero arguments, it C<croak()>s.
+Use the promise chaining instead:
 
-  $db->multi_p(
-    $db->set_p("x:y:z" => 1011),
-    $db->get_p("x:y:z"),
-    $db->incr_p("x:y:z"),
-    $db->incrby_p("x:y:z" => -10),
-  )->then(sub {
-  });
+  $db->multi_p->then(sub {
+    Mojo::Promise->all(
+      $db->set_p(...),
+      $db->incr_p(...),
+      ...
+    );
+  })->then(sub {
+	$db->exec_p;
+  })->then ...
 
 See L<https://redis.io/commands/multi> for more information.
 
