@@ -113,12 +113,20 @@ sub _on_response {
   my ($self, $conn, $res) = @_;
   $self->emit(reconnect => $conn) if delete $self->{reconnect};
 
+  # $res = [pmessage => $name, $channel, $data]
+  # $res = [message  =>        $channel, $data]
+
   return                    unless ref $res eq 'ARRAY';
   return $self->emit(@$res) unless $res->[0] =~ m!^p?message$!i;
-  my ($psub) = $res->[0] eq 'pmessage' ? splice @$res, 1, 1 : ();
-  $res->[2] = eval { from_json $res->[2] } if $self->{json}{$res->[1]};
-  my $keyspace_listen = $self->{keyspace_listen}{$psub || $res->[1]};
-  for my $cb (@{$self->{chans}{$psub || $res->[1]}}) { $self->$cb($keyspace_listen ? [@$res[1, 2]] : $res->[2]) }
+
+  my ($name)          = $res->[0] eq 'pmessage' ? splice @$res, 1, 1 : ($res->[1]);
+  my $keyspace_listen = $self->{keyspace_listen}{$name};
+
+  local $@;
+  $res->[2] = eval { from_json $res->[2] } if $self->{json}{$name};
+  for my $cb (@{$self->{chans}{$name} || []}) {
+    $self->$cb($keyspace_listen ? [@$res[1, 2]] : $res->[2], $res->[1]);
+  }
 }
 
 sub _reconnect {
@@ -150,8 +158,8 @@ Mojo::Redis::PubSub - Publish and subscribe to Redis messages
   my $pubsub = $redis->pubsub;
 
   $pubsub->listen("user:superwoman:messages" => sub {
-    my ($pubsub, $message) = @_;
-    say "superwoman got a message: $message";
+    my ($pubsub, $message, $channel) = @_;
+    say "superwoman got a message '$message' from channel '$channel'";
   });
 
   $pubsub->notify("user:batboy:messages", "How are you doing?");
@@ -266,7 +274,7 @@ L<Mojo::JSON/"from_json"> for a channel.
 
   # Send and receive data structures
   $pubsub->json("foo")->listen(foo => sub {
-    my ($pubsub, $payload) = @_;
+    my ($pubsub, $payload, $channel) = @_;
     say $payload->{bar};
   });
   $pubsub->notify(foo => {bar => 'I ♥ Mojolicious!'});
@@ -315,10 +323,15 @@ keyspace events and what C<@args> can be.
 
 =head2 listen
 
-  $cb = $pubsub->listen($channel => sub { my ($pubsub, $message) = @_ });
+  $cb = $pubsub->listen($channel => sub { my ($pubsub, $message, $channel) = @_ });
 
-Subscribe to a channel, there is no limit on how many subscribers a channel
-can have. The returning code ref can be passed on to L</unlisten>.
+Subscribe to an exact channel name
+(L<SUBSCRIBE|https://redis.io/commands/subscribe>) or a channel name with a
+pattern (L<PSUBSCRIBE|https://redis.io/commands/psubscribe>). C<$channel> in
+the callback will be the exact channel name, without any pattern. C<$message>
+will be the data published to that the channel.
+
+The returning code ref can be passed on to L</unlisten>.
 
 =head2 notify
 
